package model;

import java.util.*;

public class Grille {
	private final int largeur;
	private final int longueur;
	private Item[][] carte;
	private Joueur joueur;
	private Position entree;
	private Position sortie;
	private List<Position> chemin = new ArrayList<Position>();
	private List<Monstre> monstres = new ArrayList<Monstre>();
	private List<Piege> pieges = new ArrayList<Piege>();
	private List<Vie> vies=new ArrayList<Vie>();
	private boolean monstresAjoutes = false; 
	public Grille(int longueur, int largeur) {
		if ((largeur < 0) && (longueur < 0)) {
			new IllegalArgumentException();
		}
		this.largeur = largeur;
		this.longueur = longueur;
		this.carte = new Item[this.longueur][this.largeur];
		remplirGrille();
	}

	public Grille(int longueur, int largeur, String nomJoueur) {
		this(longueur, largeur);
		this.nouvJoueur(nomJoueur);
	}

	public Grille(int longueur, int largeur, Joueur joueur) {
		this(longueur, largeur);
		this.joueur = joueur;
		this.joueur.setPosition(entree);
	}

	private void remplirGrille() {
		// Remplissage des murs
		this.remplirMur();
		this.remplirObstacle();
		this.cheminPossible();
		if (!monstresAjoutes) {
            this.ajouteMonstre(5);
            monstresAjoutes = true;
        }
		this.ajoutePiege(largeur / 3);
		this.nouvJoueur("Player one");
		this.ajouteCoeur(3);


		this.ajoutePotion(2);

	}


////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////// Getters and Setters /////////////////////////////////////////////////////////////////

	public Item[][] getCarte() {
		return carte;
	}

	public void setCarte(Item[][] carte) {
		this.carte = carte;
	}

	public Joueur getJoueur() {
		return joueur;
	}

	public void setJoueur(Joueur joueur) {
		this.joueur = joueur;
	}

	public Position getEntree() {
		return entree;
	}

	public void setEntree(Position entree) {
		this.entree = entree;
	}

	public Position getSortie() {
		return sortie;
	}

	public void setSortie(Position sortie) {
		this.sortie = sortie;
	}

	public List<Position> getChemin() {
		return chemin;
	}

	public void setChemin(List<Position> chemin) {
		this.chemin = chemin;
	}

	public List<Monstre> getMonstres() {
		return monstres;
	}

	public void setMonstres(List<Monstre> monstres) {
		this.monstres = monstres;
	}

	public List<Piege> getPieges() {
		return pieges;
	}

	public void setPieges(List<Piege> pieges) {
		this.pieges = pieges;
	}
	
	public List<Vie> getVies() {
		return vies;
	}

	public void setVies(List<Vie> vies) {
		this.vies = vies;
	}

	public int getLargeur() {
		return largeur;
	}

	public int getLongueur() {
		return longueur;
	}

	@Override
	public String toString() {
		return "Grille [largeur=" + largeur + ", longueur=" + longueur + "]";
	}

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Operation sur la grille ///////////////////////////////////////////////
	public int randLarg() {
	return new Random().nextInt(largeur - 1);
}

	public int randLong() {
	return new Random().nextInt(longueur - 1);
}

	public void afficheGrilleTerminal() {
		for (int i = 0; i < longueur; i++) {
			for (int j = 0; j < largeur; j++) {
				if (carte[i][j] != null) {
					System.out.print(" " + carte[i][j].getNom() + " ");
				} else {
					System.out.print("   ");
				}
			}
			System.out.println("");
		}
			
	}
	
	public void deplacerMonstreGrille() {
		for(Monstre m: monstres) {
			m.deplacerMonstre(this);
		}
	}

	public void remplirMur() {
		for (int j = 0; j < longueur; j++) {
			for (int i = 0; i < largeur; i++) {
				carte[j][i] = null;
				if ((j == 0) || (i == 0) || (i == largeur - 1) || (j == longueur - 1)) {
					carte[j][i] = new Mur(new Position(j, i));
				}
			}
		}
	}

	public void remplirObstacle() {
		for (int j = 0; j < longueur; j++) {
			for (int i = 0; i < largeur; i++) {
				if ((j != 0) && (i != 0) && (i != largeur - 1) && (j != longueur - 1)) {
					if (Math.random() < 0.5) {
						Position p = new Position(j, i);
						carte[j][i] = new Mur(p);
					}
				}
			}
		}
	}

	public void cheminPossible() {
		int direction = 1;
		int x = 0;
		int y = randLong();
		if (y == 0) {
			y++;
		}
		if (Math.random() > 0.5) {
			direction = -1;
			x = largeur - 1;
		}
		carte[y][x] = null;
		entree = new Position(x, y);
		if (direction < 0) {
			x--;
		} else {
			x++;
		}

		while ((x < largeur - 1) && (x > 0)) {
			int yDeb = y;
			int deplace = randLong();
			if (Math.random() > 0.5) {
				deplace *= -1;
			}
			while (((y != yDeb + deplace) && ((y > 0) && (y < longueur - 1))) || ((0 == deplace) && (y == yDeb))) {
				carte[y][x] = null;
				chemin.add(new Position(x, y));
				if (deplace < 0) {
					y -= 1;
				} else {
					y += 1;
				}
			}
			x += direction;
			if (yDeb < y) {
				y -= 1;
			} else {
				y += 1;
			}
		}
		carte[y][x] = null;
		sortie = new Position(x, y);
	}

	public void ajouteMonstre(int nombre) {
		for (int i = 0; i < nombre; i++) {
			int id = new Random().nextInt(chemin.size());
			Position position = chemin.get(id);
			if (estVide(position.getY(), position.getX())) {
				Monstre monstre = new Monstre("M", position);
				carte[position.getY()][position.getX()] = monstre;
				monstres.add(monstre);
			} else {
				i--;
			}
		}
	}
	
	public void ajoutePiege(int nombre) {
		for (int i = 0; i < nombre; i++) {
			int y = new Random().nextInt(longueur);
			int x = new Random().nextInt(largeur);
			Position position = new Position(x, y);
			if (estVide(y, x)) {
				Piege piege = new Piege("P", position);
				carte[y][x] = piege;
				pieges.add(piege);
			} else {
				i--;
			}
		}
	}
	
	public void ajouteCoeur(int nombre) {
		for (int i = 0; i <= nombre; i++) {
			int y = new Random().nextInt(longueur);
			int x = new Random().nextInt(largeur);
			Position position = new Position(x, y);
			if (estVide(y, x)) {
				Vie coeur = new Vie("Coeur", position);
				carte[y][x] = coeur;
				vies.add(coeur);
			} else {
				i--;
			}
		}
	}

	public void nouvJoueur(String nom) {
		joueur = new Joueur(nom, entree);
		carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = joueur;
	}

	// -1 : rien a ete fait
	// 0 : moouvement reussi
	// 1 : attaque monstre
	// 2 : tomber dans un piege
	// 3 : entree
	// 4 : sortie
	// 5 : fin
	
	public int deplacerJoueur(int direction){
		Position pos = joueur.bougerVers(direction);
		int newRow = pos.getY();
		int newCol = pos.getX();
		if (newRow >= 0 && newRow < getLongueur() && newCol >= 0 && newCol < getLargeur()) {
	        if (estVide(newRow, newCol) && !estMur(newRow, newCol)) {
				// destruction de l'aciennne position
	            carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = null;
	            joueur.setPosition(pos);
				carte[newRow][newCol] = joueur;
				return 0;
	        } else if (estMonstre(newRow, newCol)) {
				joueur.attaquerMonstre((Monstre)carte[newRow][newCol], this);
				return 1;
	        } else if (estPiege(newRow, newCol)) {
				if(((Piege)carte[newRow][newCol]).getEtat()){
					((Piege)carte[newRow][newCol]).desactiverPiege();
					carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = null;
					joueur.setPosition(pos);
					carte[newRow][newCol] = joueur;
					joueur.perdrePointsDeVie(20);
					if(!joueur.estVivant()){
						return 5;
					}
					return 2;
				}
			}else if (estVie(newRow, newCol)){
				if(((Vie)carte[newRow][newCol]).getEtat()){
					System.out.println("Glup Glup, Je suis bieenn!!");
					((Vie)carte[newRow][newCol]).desactiverVie();
					carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = joueur;
					carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = null;
					joueur.setPosition(pos);
					carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = joueur;
					joueur.addVie();
					
				}
				return 7;
			}else if(estEntree(newRow, newCol)) {
				return 3;
			} else if(estSortie(newRow, newCol)) {
				return 4;

			} else if (estPotion(newRow, newCol)) {
				joueur.prendrePotion((Potion)carte[newRow][newCol]);
				carte[joueur.getPosition().getY()][joueur.getPosition().getX()] = null;
				joueur.setPosition(pos);
				carte[newRow][newCol] = joueur;
				// joueur.ajouteObjet(carte[newRow][newCol]);
				return 6;
	        }

	    }
		return -1;
	}

	public void ajoutePotion(int nombre) {
		for (int i = 0; i < nombre; i++) {
			int y = new Random().nextInt(longueur);
			int x = new Random().nextInt(largeur);
			Position position = new Position(x, y);
			if (estVide(y, x)) {
				Potion potion = new Potion(position);
				carte[y][x] = potion;
			} else {
				i--;
			}
		}
	}

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////// Teste sur la grille //////////////////////////////////////////////////
	public boolean estVide(int row, int col) {
		return (!estEntree(row, col) && !estSortie(row, col) && this.carte[row][col] == null);
	}

	public boolean estEntree(int row, int col) {
		return (col == entree.getX()) && (row == entree.getY());
	}

	public boolean estSortie(int row, int col) {
		return col == sortie.getX() && row == sortie.getY();
	}

	public boolean estMur(int row, int col) {
		if (!estVide(row, col) && !estEntree(row, col) && !estSortie(row, col)) {
			return carte[row][col].estUnMur();
		}
		return false;
	}

	public boolean estMonstre(int row, int col) {
		if (!estVide(row, col) && !estEntree(row, col) && !estSortie(row, col)) {
			return carte[row][col].estUnMonstre();
		}
		return false;
	}

	public boolean estPiege(int row, int col) {
		if (!estVide(row, col) && !estEntree(row, col) && !estSortie(row, col)) {
			return carte[row][col].estUnPiege();
		}
		return false;
	}

	public boolean estLeJoueur(int row, int col) {
		if ((row == this.getJoueur().getPosition().getY()) && (col == this.getJoueur().getPosition().getX())){
			return true;
		}
		return false;
	}
	public boolean estVie(int row, int col) {
		if (!estVide(row, col) && !estEntree(row, col) && !estSortie(row, col)) {
			return carte[row][col].estVie();
		}
		return false;
	}

	public boolean estPotion(int row, int col) {
		if (!estVide(row, col) && !estEntree(row, col) && !estSortie(row, col)) {
			return carte[row][col].estUnePotion();
		}
		return false;
	}
}