package view;

import controller.*;

import model.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class Fenetre extends JFrame {
    private static final long serialVersionUID = 1L;
    private Niveau niveau;
    private final int largeur;
    private final int longueur;
    private Case[][] carte;
    private JPanel map;
    private JLabel pv;
    private JLabel objets;
    private JLabel action;
    private JLabel nomNiveau;
    private JLabel monstres;
    private Timer bougeMonstre;
    private ListnerJoueur ecouteur;

    public Fenetre(Niveau niveau) {
        int padding = 10;
        setTitle("Knight Game");
        setSize(1000, 600);

        this.largeur = niveau.getLargeur();
        this.longueur = niveau.getLongueur();
        this.niveau = niveau;

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JPanel panel = new JPanel(new BorderLayout());
        map = new JPanel(new GridLayout(longueur, largeur));
        JPanel nav = new JPanel(new GridLayout(4, 1));
        nav.setBorder(new EmptyBorder(padding, padding, padding, padding));
        JPanel joueurPanel = new JPanel(new GridLayout(3, 2));
        JPanel monstrePanel = new JPanel(new BorderLayout());
        JPanel piegePanel = new JPanel(new BorderLayout());
        
        JPanel action = new JPanel(new BorderLayout());


        // Ajout de l'échiquier
        this.carte = new Case[this.longueur][this.largeur];
        panel.add(map, BorderLayout.CENTER);
        afficheGrille();

        // Ajout de la navigation des scores à droite
        nomNiveau = new JLabel("Niveau "+niveau.getNumeroNiveau());
        nomNiveau.setHorizontalAlignment(SwingConstants.CENTER);
        nav.add(nomNiveau, BorderLayout.NORTH);

        //premiere ligne
        joueurPanel.add(new JLabel("Nom : "));
        joueurPanel.add(new JLabel(niveau.getJoueur().getNom()));

        //deuxieme ligne
        joueurPanel.add(new JLabel("Vie : "));
        this.pv = new JLabel(Integer.toString(niveau.getJoueur().getVie()));
        joueurPanel.add(this.pv);

        //troisieme ligne
        joueurPanel.add(new JLabel("Sac : "));
        this.objets = new JLabel("vide");
        joueurPanel.add(this.objets);

        nav.add(joueurPanel);

        monstrePanel.add(new JLabel("Monstre: "));
        this.monstres = new JLabel(Integer.toString(Monstre.getNombreDeMonstre()));
        monstrePanel.add(this.monstres);
        nav.add(monstrePanel);

        piegePanel.add(new JLabel("Piege: "));
        this.monstres = new JLabel(Integer.toString(Piege.nombrePiege()));
        piegePanel.add(this.monstres);
        nav.add(piegePanel);
        
        

        // Ajout de l'action
        this.action = new JLabel("Action", SwingConstants.CENTER);
        action.add(this.action);

        

        // Ajout des panels à la fenêtre principale
        panel.add(nav, BorderLayout.EAST);
        panel.add(action, BorderLayout.SOUTH);
        add(panel);
        setBackground(Color.BLACK);

        panel.setBorder(new EmptyBorder(padding, padding, padding, padding));
        pack();
        setVisible(true);

        // Déplacement des monstres toutes les secondes
        Timer timer = new Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                for (Monstre monstre : niveau.getMonstres()) {
                    monstre.deplacerMonstre(niveau);
                }
                updateGrille();
            }
        });
        timer.start();
        bougeMonstre = timer;

        this.ecouteur = new ListnerJoueur(niveau, this);
    }

    public void afficheGrille() {
        map.removeAll();
        map.setLayout(new GridLayout(longueur, largeur));

        for (int row = 0; row < longueur; row++) {
            for (int col = 0; col < largeur; col++) {
                Case carre = new Case("Vide");

                if (niveau.estLeJoueur(row, col)) {
                    carre.setType("Joueur");
                } else if (niveau.estMur(row, col)) {
                    carre.setType("Mur");
                } else if (niveau.estPiege(row, col)) {
                    carre.setType("Piege");
                } else if (niveau.estVie(row, col)) {
                    carre.setType("Vie");
                } else if (niveau.estMonstre(row, col)) {
                    carre.setType("Monstre");
                } else if (niveau.estPotion(row, col)) {
                    carre.setType("Potion");
                } else if (niveau.estEntree(row, col)) {
                    carre.setType("Entree");
                } else if (niveau.estSortie(row, col)) {
                    carre.setType("Sortie");
                } else {
                    carre.setType("Vide");
                }
                carte[row][col] = carre;
                map.add(carre);
            }
        }
        map.revalidate();
        map.repaint();
    }

    public void updateGrille() {
        afficheGrille();
    }

    public int getLargeur() {
        return largeur;
    }

    public int getLongueur() {
        return longueur;
    }

    public void gameOver() {
        System.out.println("Game over");
        // bougeMonstre.stop();
    
        // map.removeAll();
        // map.revalidate();
        // map.repaint();
    
        // JPanel newMap = new JPanel();
    
        // this.remove(map);
        // this.add(newMap);
        // this.revalidate();
        // this.repaint();
    
        // map = newMap;
        this.getContentPane().remove(map);
    }
    
    public void setNiveau(Niveau niveau){
        this.niveau = niveau;
        bougeMonstre.stop();
        afficheGrille();
        Timer timer = new Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                for (Monstre monstre : niveau.getMonstres()) {
                    monstre.deplacerMonstre(niveau);
                }
                updateGrille();
            }
        });
        timer.start();
        bougeMonstre = timer;
        setNomNiveau();
        this.ecouteur.setNiveau(niveau);
    }

    //////////////////////////////////////
//Evenements pour le joueur
    public void afficheAction(String action){
        this.action.setText(action);
    }

    public void affichePv(){
        this.pv.setText(Integer.toString(niveau.getJoueur().getVie()));
    }

    public void message(String msg){
        // Utiliser SwingUtilities.invokeLater pour garantir que la boîte de dialogue est créée et affichée sur le thread de l'interface utilisateur
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                Dialogue messageFrame = new Dialogue(msg);
                messageFrame.setVisible(true);
            }
        });
    }

/////////////////////////////////////
// Modification du niveau
    public void setNomNiveau(){
        nomNiveau.setText("Niveau "+niveau.getNumeroNiveau());
    }
}
